<?php
if(!defined('ABSPATH')) die; // Die if accessed directly

class DataQlick__Main {
    private static $initiated = false;

    public static function init() {
        if ( ! self::$initiated ) {
            self::init_hooks();
        }
    }

    private static function init_hooks() {
        self::$initiated = true;

        add_action( 'init', array('DataQlick__Main', 'add_pages_for_export') );
        add_filter( 'template_include', array('DataQlick__Main', 'add_plugin_url_templates') );

        add_action( 'woocommerce_order_object_updated_props', array('DataQlick__Main', 'add_to_updated_orders'), 10, 1 );
        add_action( 'woocommerce_update_product', array('DataQlick__Main', 'add_to_updated_products'), 10, 1 );
        add_action( 'shutdown', array('DataQlick__Main', 'check_for_updated_items'), 10, 1 );
    }

    public static function add_pages_for_export() {
        self::add_plugin_routes();

        add_filter( 'query_vars', function($vars) {
            $vars[] = 'dataqlick_type';
            $vars[] = 'page';
            $vars[] = 'pagesize';
            $vars[] = 'updatedAfter';
            $vars[] = 'productid';
            $vars[] = 'token';
            return $vars;
        } );
    }

    public static function add_plugin_routes() {
        self::add_custom_rewrite_rule( 'Products' );
        self::add_custom_rewrite_rule( 'Customers' );
        self::add_custom_rewrite_rule( 'Sales' );
        self::add_custom_rewrite_rule( 'Shop' );
        self::add_custom_rewrite_rule( 'products' );
    }

    private static function add_custom_rewrite_rule( $type ) {
        add_rewrite_rule(
            '^dataqlick/'.$type.'?',
            'index.php?dataqlick_type='.$type,
            'top'
        );
    }

    public static function add_plugin_url_templates( $original_template ) {
        $page_type = get_query_var('dataqlick_type');

        try {
            if ( strlen($page_type) > 0 )
                return DATAQLICK__PLUGIN_DIR.'inc/'.strtolower($page_type).'.php';
            else
                return $original_template;
        }
        catch (Throwable $t) {
            echo 'Script Error - ',  $t->getMessage();
        }
        catch (Exception $e) {
            echo 'Script Exception - ',  $e->getMessage();
        }
    }

    public static function send_updated_orders( $customerId ) {
        $api_url = 'https://dataqlick.com/webhook/plugin/'.$customerId.'/orders/update';

        $order_ids = array_keys( $GLOBALS['dataqlick_array_of_updated_orders'] );
        $export = new DataQlick__SalesExport();
        $orders = array();

        foreach ( $order_ids as $order_id ) {
            $orders[] = $export->get_export_data_from_order_id( $order_id );
        }

        $result = array(
            'count' => count( $order_ids ),
            'page' => 1,
            'pagesize' => count( $order_ids ),
            'pagemax' => 1,
            'results' => $orders,
        );

        self::send_json_data_with_curl( $api_url, $result );
    }

    public static function send_updated_products( $customerId ) {
        $api_url = 'https://dataqlick.com/webhook/plugin/'.$customerId.'/products/update';

        $product_ids = array_keys( $GLOBALS['dataqlick_array_of_updated_products'] );
        $export = new DataQlick__ProductsExport();

        $products = $export->get_items_data( $product_ids );

        $result = array(
            'count' => count( $product_ids ),
            'page' => 1,
            'pagesize' => count( $product_ids ),
            'pagemax' => 1,
            'results' => $products,
        );

        $api_res = self::send_json_data_with_curl( $api_url, $result );
    }

    public static function add_to_updated_orders( $order) {
        $order_id = $order->get_id();
        $GLOBALS['dataqlick_array_of_updated_orders'][ $order_id ] = $order_id;
    }

    public static function add_to_updated_products( $product_id ) {
        $GLOBALS['dataqlick_array_of_updated_products'][ $product_id ] = $product_id;
    }

    public static function check_for_updated_items() {
        $customerId = get_option( 'dataqlick_customerId' );

        if ( ! $customerId ) {
            return;
        }

        try {
            if ( count( $GLOBALS['dataqlick_array_of_updated_products'] ) > 0) {
                self::send_updated_products( $customerId );
            }

            if ( count( $GLOBALS['dataqlick_array_of_updated_orders'] ) > 0) {
                self::send_updated_orders( $customerId );
            }
        }
        catch (Throwable $t) {
            echo 'Script Error - ',  $t->getMessage();
        }
        catch (Exception $e) {
            echo 'Script Exception - ',  $e->getMessage();
        }

    }

    public static function filter_data_by_modified_date( $query ) {
        $updated_after = get_query_var('updatedAfter');
        $date = strtotime( $updated_after );

        $query->query_vars['date_query'] = array(
            array(
                'column' => 'post_modified',
                'after' => array(
                    'year' => date('Y', $date),
                    'month' => date('m', $date),
                    'day' => date('d', $date),
                    'hour' => date('H', $date),
                    'minute' => date('i', $date),
                    'second' => date('s', $date),
                )
            )
        );
    }

    public static function add_plugin_action_links( $links, $file ) {
        if ( false !== strpos($file, 'dataqlick.php') ) {
            $links[] = "<a href='".admin_url('options-general.php?page=dataqlick_inventory_settings_page')."'>Settings</a>";
        }

        return $links;
    }

    public static function add_plugin_admin_menu() {
        add_options_page(
            'DataQlick plugin',
            'DataQlick',
            'manage_options',
            'dataqlick_inventory_settings_page',
            'dataqlick_inventory_settings_screen'
        );
    }

    public static function send_json_data_with_curl( $url, $data, $new_header = null ) {
        $header = array( 'Content-Type: application/json' );

        if ( $new_header ) {
            $header[] = $new_header;
        }

        $ch = curl_init( $url );
        curl_setopt( $ch, CURLOPT_CUSTOMREQUEST, "POST" );
        curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode($data) );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $header );

        $result = curl_exec($ch);

        curl_close( $ch );

        return $result;
    }

    public static function on_activation() {
        self::add_plugin_routes();
        flush_rewrite_rules();
    }

    public static function on_deactivation() {
        flush_rewrite_rules();
    }

}