<?php
if(!defined('ABSPATH')) die; // Die if accessed directly

class DataQlick__SalesExport extends DataQlick__Export {

    public function get_all_items_count() {
        return count(
            wc_get_orders(
                array( 'type' => 'shop_order', 'limit' => -1, 'return' => 'ids' )
            )
        );
    }

    public function get_items_data() {
        $orders = wc_get_orders( array(
            'return' => 'ids',
            'type' => 'shop_order', // need this field, because default this function returns shop_order && shop_order_refund
            'limit' => $this->pagesize,
            'offset' => ($this->page - 1) * $this->pagesize,
        ) );

        $result = array();

        foreach ( $orders as $order_id ) {
            $result[] = $this->get_export_data_from_order_id( $order_id );
        }

        return $result;
    }

    public function get_export_data_from_order_id( $order_id ) {
        $this->order = wc_get_order( $order_id );

        $customer_id = $this->order->get_customer_id();
        $customer = get_userdata( $customer_id );
        $order_items = $this->get_order_items();

        return array(
            'id' => $order_id,
            'docNumber' => (string)$order_id,
            'shippingCost' => (float)$this->order->shipping_total,
            'currency' => $this->order->currency,

            'discounts' => $this->get_order_discounts(),
            'taxes' => $this->get_order_taxes(),
            'items' => $order_items,
            'customer' => DataQlick__CustomersExport::get_export_data_from_user( $customer ),

            'payments' => $this->get_order_payments(),
            'refunds' => $this->get_order_refunds(),
            'shipments' => $this->get_order_shipments( $order_items ),

            'updated' => $this->order->date_modified->__toString(),
            'date' => $this->order->date_created->__toString(),
            'dueDate' => $this->order->date_created->__toString(),
        );
    }

    private function get_order_items() {
        $items = $this->order->get_items();
        $result = array();


        foreach ( $items as $order_item ) {

            $item_data = $order_item->get_data();

            if ( $item_data['variation_id'] ) {
                $product = wc_get_product( $item_data['variation_id'] );
                $product_id = $item_data['variation_id'];
            }
            else {
                $product =  $order_item->get_product();
                $product_id = $item_data['product_id'];
            }

            if ( ! $product ) {
                continue;
            }

            $taxes_id = key( $item_data['taxes']['total'] );

            $taxes = ( $taxes_id ? array( 'id' => $taxes_id ) : null );

            $result[] = array(
                'lineId' => $product_id,
                'itemId' => $product_id,
                'qty' => $item_data['quantity'],
                'price' => (float)$product->get_price(),
                'isTaxable' => $product->is_taxable(),
                'discounts' => $this->get_order_item_discounts( $product_id ),
                'taxes' => ( $taxes ? array( $taxes ) : array() ),
            );
        }

        return $result;
    }

    private function get_order_item_discounts( $product_id ) {
        $codes = $this->order->get_used_coupons();
        $product = wc_get_product( $product_id );
        
        $result = array();

        foreach ( $codes as $code ) {

            $coupon_id = wc_get_coupon_id_by_code( $code );
            $coupon = new WC_Coupon( $coupon_id );
            if ( $coupon->is_valid_for_product($product) ) {
                $result[] = array('id'=> $coupon_id);
            }
        }

        return $result;
    }

    private function get_order_discounts() {
        $codes = $this->order->get_used_coupons();

        if ( count($codes) == 0 ) {
            return array();
        }

        $result = array();
        foreach ( $codes as $code ) {

            $coupon_id = wc_get_coupon_id_by_code( $code );
            $coupon = new WC_Coupon( $coupon_id );

            $coupon_data = $coupon->get_data();

            $result[] = array(
                'id' => $coupon_id,
                'name' => $coupon_data['description'],
                'discountValue' => (float)$coupon_data['amount'],
                'discountType' => ( $coupon_data['discount_type'] === 'percent' ? 'Percent' : 'Sum' ),
                'applyToAllItems' => $this->is_discount_apply_to_all_items( $coupon_data ),
            );

        }

        return $result;
    }

    private function is_discount_apply_to_all_items( $data ) {
        if ( $data['product_ids'] 
            || $data['excluded_product_id']
            || $data['product_categories']
            || $data['excluded_product_categories']
            ||$data['exclude_sale_items'] ) {
            return false;
        }

        return true;
    }

    private function get_order_taxes() {
        $taxes = $this->order->get_data()['tax_lines'];
        $result = array();

        foreach ( $taxes as $tax ) {

            $tax_data = $tax->get_data();
            $tax_id = $tax_data['rate_id'];
            $tax_rate = (float)WC_Tax::_get_tax_rate( $tax_id )['tax_rate'];

            $result[] = array(
                'id' => $tax_id,
                'name' => $tax_data['label'],
                'rate' => $tax_rate,
            );
        }
        
        return $result;
    }

    private function get_order_payments() {
        return array(
            'id' => $this->order->id,
            'type' => $this->order->payment_method,
            'amount' => (float)$this->order->total,
        );
    }

    private function get_order_refunds() {
        $refunds = $this->order->get_refunds();
        $result = array();

        foreach ( $refunds as $refund ) {
            $refund_data = $refund->get_data();
            $result[] = array(
                'id' => $refund_data['id'],
                'restock' => true,
                'items' => $this->get_order_refunds_items( $refund_data ),
                'date' => $refund_data['date_created']->__toString(),
            );
        }

        return $result;
    }

    private function get_order_refunds_items( $refund_data ) {
        $result = array();

        if ( count( $refund_data['line_items']) === 0 ) { //refunded all items in order
            $items = $this->order->get_items();
            $refund_data['line_items'] = $items;
        }

        foreach ( $refund_data['line_items'] as $item ) {
            $result[] = array(
                'lineId' => ( $item['variation_id'] ? (int)$item['variation_id'] : (int)$item['product_id'] ),
                'qty' => abs( $item['quantity'] ),
            );
        }

        return $result;
    }

    private function get_order_shipments( $order_items ) {
        $result = array();
        $shippings = $this->order->get_shipping_methods();

        foreach ($shippings as $ship) {

            $result[] = array(
                'id' => (string)$this->order->id,
                'date' => $this->order->date_modified->__toString(),
                'price' => (float)$this->order->total,
                'items' => $this->get_order_shipments_items( $order_items ),
                'method' => $ship->get_data()['name'],
                'trackingNumber' => '',
            );
        }

        return $result;
    }   
        
    private function get_order_shipments_items( $items ) {
        $result = array();

        foreach( $items as $item ) {
            $result[] = array(
                'lineId' => $item['lineId'],
                'qty' => $item['qty']
            );
        }

        return $result;
    }

}