<?php
if(!defined('ABSPATH')) die; // Die if accessed directly

abstract class DataQlick__Export {

	abstract protected function get_all_items_count();
	abstract protected function get_items_data();

    const DEFAULT_PAGE_NUMBER = 1;
    const DEFAULT_PAGE_SIZE = 10;

    protected $page;
    protected $pagesize;

    private function check_token() {
        $token = get_query_var( 'token' );
        $shop_token = get_option( 'dataqlick_shop_token' );

        return $token === $shop_token;
    }

    protected function init_variables() {
        if ( ! $this->check_token() ) {
            header('HTTP/1.0 401 Unauthorized');
            die( 'access denied for this token' );
        }

        $this->page = get_query_var( 'page' );
        $this->page = $this->page ? $this->page : self::DEFAULT_PAGE_NUMBER;

        $this->pagesize = get_query_var( 'pagesize' );
        $this->pagesize = $this->pagesize ? $this->pagesize : self::DEFAULT_PAGE_SIZE;

        if ( get_query_var( 'updatedAfter' ) ) {
            add_action( 'parse_query', array( 'DataQlick__Main', 'filter_data_by_modified_date' ) );
        }
    }

    public function display_json_data() {
        $this->init_variables();

        $total_items_count = $this->get_all_items_count();
        $pagemax = ceil( $total_items_count / $this->pagesize );

        $result = array(
            'count' => $total_items_count,
            'page' => $this->page,
            'pagesize' => (int)$this->pagesize,
            'pagemax' => $pagemax,
            'results' => $this->get_items_data(),
        );

        header('Content-Type: application/json');
        echo json_encode( $result, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
    }

}