<?php
if(!defined('ABSPATH')) die; // Die if accessed directly

add_filter( 'plugin_action_links', array( 'DataQlick__Main', 'add_plugin_action_links' ), 10, 2 );
add_action( 'admin_menu', array( 'DataQlick__Main', 'add_plugin_admin_menu') );

function dataqlick_get_shop_register_data( $data) {

    $register_api_url = DATAQLICK_REGISTRATION_SHOP_API_URL;
    $auth_header = "Authorization: {$data['token_type']} {$data['access_token']}";

    $shop_token = bin2hex(openssl_random_pseudo_bytes(16));
    $GLOBALS['dataqlick_register_shop_data']['dataqlick_shop_token'] = $shop_token;

    $shop_name = get_bloginfo('name');
    $shop_currency = apply_filters( 'woocommerce_currency', get_option( 'woocommerce_currency' ) );

    $data = array(
        'providerId' => 'woocommerce',
        'url' => get_site_url().'/dataqlick',
        'token' => $shop_token,
        'shopName' => $shop_name,
        'currency' => $shop_currency,
    );

    return DataQlick__Main::send_json_data_with_curl( DATAQLICK_REGISTRATION_SHOP_API_URL, $data, $auth_header );
}

function dataqlick_inventory_settings_screen() {
    $user = get_option('dataqlick_user');
    $user = $user ? $user : '';

    $pass = get_option('dataqlick_pass');
    $pass = $pass ? $pass : '';

    $barcode = get_option('dataqlick_barcode');
    $barcode = $barcode ? $barcode : '';

    $dataqlick_error_message = '';

    if ( isset($_POST['submit']) && check_admin_referer('nonce_action__dataqlick', 'nonce_field__dataqlick') && current_user_can('manage_options') ) {

        $_POST['barcode'] = sanitize_text_field( $_POST['barcode'] );
        $_POST['user'] = sanitize_text_field( $_POST['user'] );

        update_option( 'dataqlick_barcode', $_POST['barcode'] );

        $GLOBALS['dataqlick_register_shop_data'] = array(
            'dataqlick_user' => $_POST['user'],
            'dataqlick_pass' => $_POST['pass'],
            'dataqlick_shop_token' => '',
            'dataqlick_customerId' => '',
        );

        $data = array(
            "email" => $_POST['user'],
            "password" => $_POST['pass']
        );

        $login_data = DataQlick__Main::send_json_data_with_curl( DATAQLICK_LOGIN_API_URL, $data );
        $login_data = json_decode($login_data, true);

        if ( $login_data['token_type'] && $login_data['access_token'] ) {

            $register_api_data = dataqlick_get_shop_register_data( $login_data );
            $register_api_data = json_decode($register_api_data, true);

            if ( $register_api_data['customerId'] ) {
                $GLOBALS['dataqlick_register_shop_data']['dataqlick_customerId'] = (int)$register_api_data['customerId'];
            }
            elseif( $register_api_data['error'] ) {
                $dataqlick_error_message = 'DataQlick Registration Shop API error: '.$register_api_data['error'];
            }
            else {
                 $dataqlick_error_message = 'Can\'t read data from DataQlick Registration Shop API.';
            }

        } else {
            $dataqlick_error_message = 'Can\'t read data from DataQlick Login API.';
        }

        if ( ! $dataqlick_error_message ) {
            foreach ( $GLOBALS['dataqlick_register_shop_data'] as $key => $value ) {
               update_option( $key, $value );
            }
        }
    }

    ?>
    
    <style>
        .dataqlick-title-block img { float: left; height: 35px; }
        .dataqlick-title-block h2 { float: left; margin: 0; line-height: 35px; margin-left: 10px; }
        .dataqlick-clearfix { clear: both; }
        .dataqlick-title-message { font-size: 16px; margin-top: 20px; }
        .dataqlick-separator-line { border: 1px solid #adadad; margin: 30px 0; }
        .dataqlick-section-number { background: red; padding: 5px; border-radius: 30px; color: white; width: 8px; font-weight: bold;
            height: 8px; line-height: 8px; text-align: center; float: left; margin-right: 10px; margin-top: 1px; font-size: 14px; }
        .dataqlick-section-title { font-weight: bold; font-size: 20px; margin-bottom: 50px; }
        .dataqlick-attributes-block { text-align: center; margin-bottom: 30px;}
        .dataqlick-inline-block { display: inline-block; }
        .dataqlick-attr-text { float: left; text-align: right; width: 140px; margin-right: 20px; padding-top: 11px;  }
        .dataqlick-attr-top-text { font-weight: bold; font-size: 15px; }
        .dataqlick-attr-bot-text { font-size: 12px; }
        .dataqlick-action-block { float: left; width: 390px; text-align: left; }
        .dataqlick-action-block select, .dataqlick-action-block input { width: 100%; font-size: 15px; height: 35px; }
        .dataqlick-success-info { color: #3c763d; background-color: #dff0d8; padding: 15px; margin-top: 20px; font-size: 16px;
            border: 1px solid #d6e9c6; border-radius: 4px; }
        .dataqlick-error-info {  color: #a94442; background-color: #f2dede; padding: 15px; margin-top: 20px; font-size: 16px;
            border: 1px solid #ebccd1; border-radius: 4px; }
    </style>

    <div class="wrap">
        <div class="dataqlick-settings-form">
            <form method="POST" onsubmit="return validateMyForm();" autocomplete="off">

                <?php wp_nonce_field('nonce_action__dataqlick', 'nonce_field__dataqlick'); ?>

                <div class="dataqlick-title-block">
                    <img src="https://cdn.shopify.com/s/files/applications/71d63d165dd131cd051bd54f10bd99bd_512x512.png?1506126829">
                    <h2>DATAQLICK CONFIGURATIONS</h2>
                </div>

                <div class="dataqlick-clearfix"></div>
                <div class="dataqlick-title-message">
                    Record your sales into accounting platform, manage inventory across multiple sales channels, simplify your work.
                    <br>Create DataQlick account online
                </div>

                <?php
                    if ( isset($_POST['submit']) ) {
                        if ( $dataqlick_error_message ) {
                            echo "<div class=\"dataqlick-error-info\">{$dataqlick_error_message}</div>";
                        } else {
                            echo "<div class=\"dataqlick-success-info\">The configuration has been saved.</div>";
                        }
                    }
                ?>

                <div class="dataqlick-separator-line"></div>
                <div class="dataqlick-section-title" style="margin-bottom: 30px;">
                    <span class="dataqlick-section-number">1</span>
                    Create DataQlick account online
                </div>
                <div class="dataqlick-title-message">If you do not have DataQlick account please create one now here:</div>
                <a href="https://www.dataqlick.com/registration/signup?utm_source=woocommerce" target="_blank">https://www.dataqlick.com/registration/signup</a>
                <div class="dataqlick-separator-line"></div>

                <div class="dataqlick-section-title">
                    <span class="dataqlick-section-number">2</span>Please enter your DataQlick credentials
                </div>

                <div class="dataqlick-attributes-block">
                    <div class="dataqlick-inline-block">
                        <div class="dataqlick-attr-text">
                            <div class="dataqlick-attr-top-text">Username</div>
                            <div class="dataqlick-attr-bot-text">[global]</div>
                        </div>

                        <div class="dataqlick-action-block">
                            <div>
                                <input type="text" name="user" autocomplete="off" value="<?php echo esc_attr($user); ?>">
                            </div>
                            <div class="dataqlick-attr-bot-text">
                                Need a DataQlick account? <a href="https://www.dataqlick.com/registration/signup?utm_source=woocommerce" target="_blank">Click here to get one</a>
                            </div>
                        </div>
                    </div>
                </div>

                <div class="dataqlick-attributes-block">
                    <div class="dataqlick-inline-block">
                        <div class="dataqlick-attr-text">
                            <div class="dataqlick-attr-top-text">Password</div>
                            <div class="dataqlick-attr-bot-text">[global]</div>
                        </div>

                        <div class="dataqlick-action-block">
                            <div>
                                <input type="password" name="pass" autocomplete="off" value="<?php echo esc_attr($pass); ?>">
                            </div>
                        </div>
                    </div>
                </div>

                <div class="dataqlick-separator-line"></div>

                <div class="dataqlick-section-title">
                    <span class="dataqlick-section-number">3</span>Attributes
                </div>

                <div class="dataqlick-attributes-block">
                    <div class="dataqlick-inline-block">
                        <div class="dataqlick-attr-text">
                            <div class="dataqlick-attr-top-text">Barcode attribute</div>
                            <div class="dataqlick-attr-bot-text">[global]</div>
                        </div>

                        <div class="dataqlick-action-block">
                            <div>
                                <input type="text" name="barcode" autocomplete="off" value="<?php echo esc_attr($barcode); ?>">
                            </div>
                            <div class="dataqlick-attr-bot-text">
                                Enter name of custom field to link UPC Barcode information in your account in DataQlick
                            </div>
                        </div>
                    </div>
                </div>

                <div>
                    <input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes">
                </div>
            </form>
        </div>
    </div>
    <script>
        function validateMyForm() {

            var submit = true;
            var user = jQuery('[name="user"]').val();
            var pass = jQuery('[name="pass"]').val();

            if ( user.length == 0 ) {
                jQuery('[name="user"]').css('border-color','#f98787')
                submit = false;
            }

            if ( user.length == 0 ) {
                jQuery('[name="pass"]').css('border-color','#f98787')
                submit = false;
            }

            return submit;
        }

        jQuery(function($) {
            $('[name="user"], [name="pass"]').change(function() {
                if ( this.value.length == 0 ) {
                    $(this).css('border-color','#f98787');
                } else {
                    $(this).css('border-color','#ddd');
                }
            });
        });

    </script>
    <?php
}
